
from IPython.display import HTML, display

from ._config import NAME, TYPE, DEFAULT, DESCRIPTION
from ._state import State, state_VIS3D



class Wrapper(object):

    def __init__(self, path=''):
        self.path = path
        self.state = state_VIS3D
        

    def __getattr__(self, item):
        _path = item if not self.path else self.path+'.'+item
        for attr in self.state._OPTION:
            if _path == attr.get(NAME):
                var = Wrapper(_path)
                if len(dir(var)):
                    setattr(self, item, var)
                # setattr(self, item, var if len(dir(var)) else '')
                return var


    def __dir__(self):
        _dir = []
        _path = self.path.split('.')
        if len(_path)==1 and self.path is '':
            for attr in self.state._OPTION:
                if len(attr.get(NAME).split('.')) == 1:
                    _dir.append(attr.get(NAME))
        else:
            for attr in self.state._OPTION:
                _attr_path = attr.get(NAME).split('.')
                if len(_path) < len(_attr_path):
                    if _path[len(_path)-1] == _attr_path[len(_path)-1] \
                            and len(_attr_path) == len(_path)+1:
                        _dir.append(_attr_path[len(_path)])
        return _dir


    def to_dict(self):
        _dic = dict(self.__dict__)
        _dic.pop('path')

        for k in list(_dic.keys()):
            if isinstance(_dic[k], list):
                for i, e in enumerate(_dic[k]):
                    if isinstance(e, Wrapper):
                        _dic[k][i] = _dic[k][i].to_dict()
                    elif isinstance(e, dict):
                        pass
                    elif _dic[k][i]=={} or _dic[k][i]==None or isinstance(_dic[k][i], State):
                        _dic[k].pop(i) 
            else:
                if isinstance(_dic[k], Wrapper):
                    _dic[k] = _dic[k].to_dict()
                elif isinstance(_dic[k], dict):
                    pass
                elif _dic[k]=={} or _dic[k]==None or isinstance(_dic[k], State):
                    _dic.pop(k) 

        return _dic


    @property
    def __doc__(self):
        for attr in self.state._OPTION:
            if self.path == attr.get(NAME):
                _type = attr.get(TYPE)
                _default = attr.get(DEFAULT)
                _desc = attr.get(DESCRIPTION)
                break

        doc = "Documentation for '%s'\n\n" % self.path + \
                   "Type\n%s\n\n" % _type + \
                   "Default\n%s\n\n" % _default + \
                   "Description\n%s\n\n" % _desc
 
        return doc


    def info(self):
        if self.path !='':
            for attr in self.state._OPTION:
                if self.path == attr.get(NAME):
                    _type = attr.get(TYPE)
                    _default = attr.get(DEFAULT)
                    _desc = attr.get(DESCRIPTION)
                    break

        doc = "<h4> Documentation for '%s' </h4><br>" % self.path + \
                "<li>Type </li>%s<br><br>" % _type + \
                "<li>Default </li>%s<br><br>" % _default + \
                "<li>Description </li>%s<br><br>" % _desc

        display(HTML(doc))






JS_JSON_PARSE_OPTION = """
var options = JSON.stringify(options);

var options = JSON.parse(options, function(key, value) {

    if (value && (typeof value==="string")) {

        if (value.substr(0,8) == "function") {
            var startBody = value.indexOf('{') + 1;
            var endBody = value.lastIndexOf('}');
            var startArgs = value.indexOf('(') + 1;
            var endArgs = value.indexOf(')');

            return new Function(value.substring(startArgs, endArgs),
                                value.substring(startBody, endBody));
        }


        if (value.substr(0,9)=="(function") {
            var startBody = value.indexOf('{') + 1;
            var endBody = value.lastIndexOf('}');
            var startArgs = value.indexOf('(', 1) + 1;
            var endArgs = value.indexOf(')');

            var func = new Function(value.substring(startArgs, endArgs),
                                    value.substring(startBody, endBody));
            return func();
        }
    }

    return value;
});
"""


JS_JSON_PARSE_DATA = """
var data = JSON.parse(data);
"""

JS_CREATE_DATASET = """
var dataset = new vis.DataSet();
e = data[0];
if (e.style!==undefined & e.filter!==undefined) {
    for (var i=0; i<data.length; i++) {
        dataset.add( {x: data[i].x, y: data[i].y, z: data[i].z, style: data[i].style, filter: data[i].filter});    
    }
}
else if (e.style!==undefined & e.filter===undefined) {
    for (var i=0; i<data.length; i++) {
        dataset.add( {x: data[i].x, y: data[i].y, z: data[i].z, style: data[i].style});    
    }
}
else if (e.style===undefined & e.filter!==undefined) {
    for (var i=0; i<data.length; i++) {
        dataset.add( {x: data[i].x, y: data[i].y, z: data[i].z, filter: data[i].filter});    
    }
}
else if (e.style===undefined & e.filter===undefined) {
    for (var i=0; i<data.length; i++) {
        dataset.add( {x: data[i].x, y: data[i].y, z: data[i].z});    
    }
}
"""



import os
import json
from ._config import VIS3D_OPTION_FILE, VIS3D_METHOD_FILE, VIS3D_DATASET_FILE, \
                    API_DIR


def load_resource(src):
    _dir = os.path.dirname(__file__)
    object_builder_file = os.path.join(_dir, API_DIR, src)
    with open(object_builder_file, encoding='UTF-8') as obj_build:
        return json.load(obj_build)


class State(object):
    """
    Contains all 'ezvis3d' API, options and methods
    """

    def __init__(self):
        self._OPTION = load_resource(VIS3D_OPTION_FILE)
        self._METHOD = load_resource(VIS3D_METHOD_FILE)
        self._DATASET = load_resource(VIS3D_DATASET_FILE)

state_VIS3D = State()





import os
import pandas as pd
import datetime as dt
import uuid
from IPython.display import HTML


from ._config import JS_LIBS_ONE, CSS_LIBS_ONE, JS_SAVE
from .scripts import JS_JSON_PARSE_OPTION, JS_JSON_PARSE_DATA


def html(data, options, center=False, save=False,
         save_name=None, save_path='saved', dated=True, notebook=True):
    """
    save=True will create a standalone HTML doc under localdir/saved (creating folfer save if necessary)
    center=True will center the plot in the output cell, otherwise left-aligned by default.
    """

    def json_dumps(obj):
        return pd.io.json.dumps(obj)

    _options = dict(options)
    _data = data

    def clean_function_str(key, n=15):
        """
        Remove new line characters in the first say 15 characters
        of the value corresponding to key in dictionary _options.
        This value is a string that potentially starts with
        'function' or ' function' or '[newline]function' or '[newline]function', etc
        This cleaning makes js string parsing easier to recognize functions.
        """
        if key in _options.keys():
            new_str = _options[key][:n].replace('\n', '')
            new_str = new_str.replace('\r', '')
            new_str = new_str + _options[key][n:]
            _options[key] = new_str

    clean_function_str('tooltip')
    clean_function_str('xValueLabel')
    clean_function_str('yValueLabel')
    clean_function_str('zValueLabel')

    chart_id = str(uuid.uuid4()).replace('-', '_')

    js_init = """
    var options = %s;
    %s
    window.opt = jQuery.extend(true, {}, options);
    console.log('vis3d options accessible as opt');
    
    var data = %s;
    %s
    window.dta = jQuery.extend(true, [], data);
    console.log('vis3d data accessible as dta');
    """ % (json_dumps(_options), JS_JSON_PARSE_OPTION,
           json_dumps(_data), JS_JSON_PARSE_DATA)

    js_call = """
    var container = document.getElementById('%s');
    console.log(options);
    console.log(data);
    graph3d = new vis.Graph3d(container, data, options);
    """ % (chart_id)

    if center:
        if not 'width' in _options.keys():
            # Explicitly sets width to default value
            _options['width'] = '750px'

        html = """
        <div id="%s" style="margin: auto; width: %s;"></div>
        """ % (chart_id, _options['width'])
    else:
        html = """
        <div id="%s"></div>
        """ % (chart_id)

    css = """
    <link href="%s" rel="stylesheet" type="text/css" />
    """ % (CSS_LIBS_ONE)

    js = """<script>
    // the Jupyter notebook loads jquery.min.js  and require.js and at the top of the page
    // then to make jquery available inside a require module
    // the trick is http://www.manuel-strehl.de/dev/load_jquery_before_requirejs.en.html
    require(%s, function(lib) {
        window.vis = jQuery.extend(true, {}, lib);
        %s
        %s
    });
    </script>""" % (JS_LIBS_ONE, js_init, js_call)

    # save
    js_load = ''.join(['<script src="%s"></script>' % e for e in JS_SAVE])
    if save == True:
        if not os.path.exists(save_path):
            os.makedirs(save_path)
        tag = save_name if save_name else 'plot'
        dated = dt.datetime.now().strftime('_%Y%m%d_%H%M%S') if dated else ''

        with open(os.path.join(save_path, tag + dated + '.html'), 'w') as f:
            f.write(js_load + html + js)

    if notebook:
        return html + js
    else:
        return js_load + html + js

    return html + css + js


def plot(data, options, center=False, save=False,
         save_name=None, save_path='saved', dated=True, notebook=True):
    contents = html(data, options, center, save=save,
                    save_name=save_name, save_path=save_path, dated=dated, notebook=notebook)
    return HTML(contents)




from ._wrapper import Wrapper
from ._plot import plot, html


class Vis3d(Wrapper):
    """
    Main vis3d Object

    Examples: http://visjs.org/graph3d_examples.html
    Docs: http://visjs.org/docs/graph3d/
    """

    def __init__(self):
        Wrapper.__init__(self)

        # default config
        # self.width = '600px'
        # self.height = '600px'

    def plot(self, df_data, center=False, save=False,
             save_name=None, save_path='saved', dated=True, notebook=True):
        "df_data format is a dataframe with columns x, y, z (required), and style, filter (optional)"
        data = df_data.to_json(orient='records')
        options = self.to_dict()
        return plot(data, options, center=center, save=save,
                    save_name=save_name, save_path=save_path, dated=dated, notebook=notebook)

    def html(self, df_data, center=False, save=False,
             save_name=None, save_path='saved', dated=True, notebook=True):
        "df_data format is a dataframe with columns x, y, z (required), and style, filter (optional)"
        data = df_data.to_json(orient='records')
        options = self.to_dict()
        return html(data, options, center=center, save=save,
                    save_name=save_name, save_path=save_path, dated=dated, notebook=notebook)




import json
from IPython.display import HTML, display



JS_LIBS_ONE = json.dumps([
    # 'https://cdnjs.cloudflare.com/ajax/libs/vis/4.7.0/vis.js',
    'https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.js'
])

JS_LIBS_ONE_sep = ';'.join(map(lambda x: 'console.log("'+x+'")', json.loads(JS_LIBS_ONE)))+';'

# CSS_LIBS_ONE = 'https://cdnjs.cloudflare.com/ajax/libs/vis/4.7.0/vis.css'
CSS_LIBS_ONE = 'https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.css'


JS_LOAD = """<script>
require(%s, function(lib) {
    console.log("The following js libs loaded:");
    %s
    window.vis = jQuery.extend(true, {}, lib);
    console.log('vis js lib accessible as vis');

});
</script>""" % (JS_LIBS_ONE, JS_LIBS_ONE_sep)


def load_js_libs():
	display(HTML(JS_LOAD))



JS_SAVE = [
    'https://cdnjs.cloudflare.com/ajax/libs/require.js/2.1.17/require.js',
    'https://cdnjs.cloudflare.com/ajax/libs/jquery/2.1.4/jquery.min.js',
]


API_DIR = 'api'
SCRIPTS_DIR = 'scripts'

VIS3D_OPTION_FILE = 'ezvis3d_option.json'
VIS3D_METHOD_FILE = 'ezvis3d_method.json'
VIS3D_DATASET_FILE = 'ezvis3d_dataset.json'

NAME = 'Name'
TYPE = 'Type'
DEFAULT = 'Default'
DESCRIPTION = 'Description'






__name__ = 'ezvis3d'
name_url = __name__.replace('_', '-')

__packages__ = [__name__]
__version__ = '0.2.3'
__description__ = 'easy vis.js Graph3d js library wrapper, accessible from pandas dataframes in the IPython notebook'
__long_description__ = 'See github repo'
__author__ = 'oscar6echo'
__author_email__ = 'olivier.borderies@gmail.com'
__url__ = 'https://github.com/{}/{}'.format(__author__,
                                            name_url)
__download_url__ = 'https://github.com/{}/{}/tarball/{}'.format(__author__,
                                                                name_url,
                                                                __version__)
__keywords__ = ['vis.js', 'Graph3d', 'pandas', 'notebook', 'javascript']
__license__ = 'MIT'
__classifiers__ = ['Development Status :: 4 - Beta',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'
                   ]
__include_package_data__ = True
__package_data__ = {
    'api':
    ['api/ezvis3d_dataset.json',
     'api/ezvis3d_dataset.tsv',
     'api/ezvis3d_method.json',
     'api/ezvis3d_method.tsv',
     'api/ezvis3d_option.json',
     'api/ezvis3d_option.tsv'
     ]
}
